import sys
import os
from cdo import *
import copy
from Scientific.N import *
from Scientific.IO.NetCDF import *
from dateutil.relativedelta import *

class Data:
    """This classe define the set of data for the observation or model
    on which will be calculate the statistics.
    The variables are the following:
    - datanames: names of experiments or observational set (string)
    - varname: name of the variables on which the statitics must be calculated (string)
    - inputdir: directory where are stored the original data (string)
    - outpudir: directory to write the output data (string)
    - filelist: list of file on which the calculations must be done (list of string)
    - sdates: list of startdate associated to each file (list of date)
    - filetype: type of the files in Filelist (string)
    - leadtime: number of timestep in each file (int)
    - freq: frequency between 2 timesteps in hours (int)
    - calendar: type of calendar in the files of filelist
    - original: if the file in file list are originals file or temporary files (True/False)"""
    def __init__(self, dataname, varname):
        self._dataname=dataname
        self._varname=varname
        self._inputdir=""
        self._outpudir=""
        self._filelist=[]
        self._sdates=[]
        self._filetype=""
        self._leadtime=-1
        self._freq=-1

    ######## set ########

    def set_dataname(self, dataname):
        self._dataname=dataname

    def set_varname(self, varname):
        self._varname=varname
   
    def set_inputdir(self, inputdir):
        self._inputdir=inputdir

    def set_outputdir(self, outputdir):
        self._outputdir=outputdir

    def set_filelist(self, filelist):
        self._filelist=filelist

    def set_sdates(self, sdates):
        self._sdates=sdates

    def set_filetype(self, filetype):
        self._filetype=filetype

    def set_leadtime(self,leadtime):
        self._leadtime=leadtime

    def set_freq(self,freq):
        self._freq=freq

    ######## get ########

    def get_dataname(self):
        return self._dataname

    def get_varname(self):
        return self._varname
   
    def get_inputdir(self):
        return self._inputdir

    def get_outputdir(self):
        return self._outputdir

    def get_filelist(self):
        return self._filelist

    def get_sdates(self):
        return self._sdates

    def get_filetype(self):
        return self._filetype

    def get_leadtime(self):
        return self._leadtime

    def get_freq(self):
        return self._freq

    ######## set_outputdirAuto ########

    def set_outputdirAuto(self):
        """look for input and output directory corresponding to the dataname"""
        inputdir=self.get_inputdir()
        if "/" in  inputdir:
            #remove the last part of the input directory and replace it by statistics
            outputdir="/".join(inputdir.split("/")[:-3])+"/"
        else:
            print "The path specify must be an abolute path"
            print "this path is not correct or is empty:"
            print inputdir
            sys.exit(0)
        self.set_outputdir(outputdir)
        if (not os.path.isdir(outputdir)):
            os.mkdir(outputdir)
            print "Created directory:"
            print outputdir
            
    def monthly_percentile(self, percentile):
        """calculate the monthly percentile for all file in filelist"""

        cdo=Cdo()
        
        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        
        for i in range(len(filelist)):
            f=filelist[i]
            fileout=(f.split("/")[-1])
            fileout=fileout.split(".")[0]+"_q"+str(percentile)+".nc"
            cdo.monpctl(percentile, input=(f,
                                           cdo.monmin(input=f,options="-r -f nc"),
                                           cdo.monmax(input=f,options="-r -f nc")), 
                        output=fileout,
                        options="-r -f nc")

            filelist[i] = fileout

        
        exp_stat=copy.deepcopy(self)
        exp_stat.set_filelist(filelist)
        #the leadtime is calculated based on the last file
        exp_stat.set_leadtime( len(cdo.showtimestamp(input=fileout)[0].split()))
        exp_stat.set_freq = relativedelta(months=+1)
        exp_stat.set_varname = varname + "_q" + str(percentile)
        return exp_stat
