import datetime
import time
from data_mod import DataMod
import sys
import os
import glob
import sets
from dateutil.relativedelta import *
from cdo import *
import copy
from Scientific.N import *
from Scientific.IO.NetCDF import *
import numpy
import shutil
from common import *



class DataEns(DataMod):
    """ classe of data for the ENSEMBLES multi-model
    """
    def __init__(self, dataname, varname):
        DataMod.__init__(self, dataname, varname)
        self._nmenber=9

    def set_inputdirAuto(self):
        """defining the path to look for the model daily data"""
        dataexp = { "EnsCmccSeas":"/cfunas/exp/ENSEMBLES/seasonal/cmcc/daily_mean/" ,
                    "EnsIfmSeas":"/cfunas/exp/ENSEMBLES/seasonal/ifm/daily_mean/",
                    "EnsMetfrSeas":"/cfunas/exp/ENSEMBLES/seasonal/metfr/daily_mean/",
                    "EnsUkmoSeas":"/cfunas/exp/ENSEMBLES/seasonal/ukmo/daily_mean/" , 
                    "EnsEcmwfSeas":"/cfunas/exp/ENSEMBLES/seasonal/ecmwf/daily_mean/"
                    } 
        #  # here add the general case when needed (m02j, m02m....)
        dataname=self.get_dataname()
        varname=self.get_varname()
        try:
            inputdir=dataexp.get(dataname) +  varname + "/"
        except:
            print "The experience " + dataname + " is not yet available in the function"
            sys.exit(0)
        
        if (not os.path.isdir(inputdir)):
            print "This directory does not exist:"
            print inputdir
            sys.exit(0)

        self.set_inputdir(inputdir)



    def format_files_nc(self):
        """this fonction check if the file in the filelist 
        have a proper format.
        get the variables leadtime and freq
        the calendar is checked and set if needed
        the variables incompatibles with cdo are removed"""

        cdo=Cdo()

        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        varname=self.get_varname()
        
        #we determine the lead time and 
        #the frequency and if it is the same in all files
        for i in range(len(filelist)):
            f=filelist[i]
            d=sdates[i]

            #test if the taxis axis is well defined
            f=settaxis(dataname, varname, f,d)
            
            taxis = (cdo.showtimestamp(input=f)[0]).split()

            #calculate leadtime and freq
            leadtime = len(taxis) 
            taxis0=time.strptime(taxis[0],'%Y-%m-%dT%H:%M:%S')
            taxis1=time.strptime(taxis[1],'%Y-%m-%dT%H:%M:%S')
            taxis0=datetime.date(taxis0.tm_year, taxis0.tm_mon, taxis0.tm_mday)
            taxis1=datetime.date(taxis1.tm_year, taxis1.tm_mon, taxis1.tm_mday)
            freq = taxis1-taxis0

            #if freq was equal too -1 set the new frequency
            if  (self.get_freq() == -1):
                self.set_freq(freq)
            elif (self.get_freq() != freq) and (self.get_freq() != -1):
                print "the file %s does not have a %s frequency" % (f, str(freq))
                sys.exit(0)


            #add here solution for cdo imcompatibilities
            

            filelist[i]=f
        self.set_filelist(filelist)
