from data import Data
#import pandas as pd
import datetime
import time
import sys
import os
import glob
import sets
from dateutil.relativedelta import *
from cdo import *
import copy
from Scientific.N import *
from Scientific.IO.NetCDF import *
import numpy
import shutil


class DataMod(Data):
    """ classe of data for the model
    -nmember: number of ensemble members
    """

    def __init__(self, dataname, varname):
        Data.__init__(self, dataname, varname)
        self._nmenber=-1
    
    def get_nmember(self):
        return self._nmenber
    
    def set_nmember(self, nmenber):
        self._nmenber = nmenber


    def set_files(self, date1, date2):
        """create the Filelist and sdates argument with all files
        between date1 and date2 separated by one year
        check if these files exist
        check their type"""
        inputdir=self.get_inputdir()
        filelist=[]
        datelist=[]

        filename=self.get_varname() + "_" + str(date1).replace("-","")
        datelist.append(date1)
        #we determine the type of file with the first year
        try:
            filename=glob.glob(inputdir+filename+'.nc')
        except:
            try:
                filename=glob.glob(inputdir+filename+".grb")
            except:
                print "the file %s does not exist" % (inputdir+filename+".nc")
                print "and the file %s does not exist" % (inputdir+filename+".grb")
                print "the directory %s must contain .nc or .grb file for the startdate %s" % (inputdir, str(date1))
                sys.exit(0)
            filetype="grb"
        filetype="nc"
        filelist.append(filename[0])
        datelist.append(date1)
        date1 = date1 + relativedelta(years=+1)
        d=date1
        d = d + relativedelta(years=+1)
        while d <= date2:
            filename=self.get_varname() + "_" + str(d).replace("-","")
            try:
                filename=glob.glob(inputdir+"/"+filename+"."+filetype)
            except:
                print "the file %s does not exit" % inputdir+filename+"*."+filetype
                print "while the file %s exists"  % inputdir+filelist[1]
                print "all file must have the same type (grb or nc)"
                sys.exit(0)
            datelist.append(d)
            filelist.append(filename[0])
            d = d + relativedelta(years=+1)

        self.set_filelist(filelist)
        self.set_sdates(datelist)


    def select_leadtime(self, leadtime1, leadtime2):
        """select the time period between leadtime1 and leadtime2
        in all files of the DataMod object:
        -leadtime1: in month (timedelta)
        -leadtime2: in month (timedelta)
        the file must have the correct format for cdo
        (run the format_files_nc method in case of failure)
        (to be changed when the right named convention has been set)
        """

        cdo=Cdo()
        
        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        freq=self.get_freq()
        
        if leadtime2 < leadtime1:
            print "the second leadtime must be larger than the first one"
            sys.exit(0)

        for i in range(len(filelist)):
            f=filelist[i]
            d=sdates[i]
            
            
            fdate1=d+leadtime1
            fdate2=d+leadtime2
            #calculate the last date of the file
            #  #done directly with the file 
            #  #to avoid the different calendar issues
            print f
            lastdate=((cdo.showtimestamp(input=f)[0]).split())[-1]
            print ((cdo.showtimestamp(input=f)[0]).split())[-1]
            lastdate=time.strptime(lastdate,'%Y-%m-%dT%H:%M:%S')
            lastdate=datetime.date(lastdate.tm_year, lastdate.tm_mon, lastdate.tm_mday)

            #check if the time period is smaller the the length of the file
            print fdate2
            print lastdate
            if (fdate2 > lastdate):
                print "the selected time period is not include in the file:"
                print f
                sys.exit(0)


            fileout=(f.split("/")[-1])
            lead1str=str(leadtime1.months).zfill(2)
            lead2str=str(leadtime2.months).zfill(2)
            fileout=fileout.split(".")[0]+"_"+lead1str+"-"+lead2str+"_01-01.nc"
            cdo.seldate(fdate1,fdate2, input=f, output=fileout,options="-r -f nc")
            print fileout
            filelist[i]=fileout
            sdates[i]=fdate1
            

        exp_season=copy.deepcopy(self)
        exp_season.set_filelist(filelist)
        exp_season.set_sdates(sdates)
        #the leadtime is calculated based on the last file
        exp_season.set_leadtime( len(cdo.showtimestamp(input=fileout)[0].split()))
        return exp_season


    def write_output(self):
        cdo=Cdo()
        
        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        freq=self.get_freq()
        leadtime=self.get_leadtime()
        nmember=self.get_nmember()

        finalvarlist = ("experiment_id", "institution", 
                        "latitude", "longitude", "leadtime"
                        "realization","reftime", "source"
                        "time_bnd")  
        vardimtable = {"experiment_id":("ensemble", "string4"),
                    "institution":("ensemble", "string15"),
                    "latitude":("latitude"),
                    "leadtime":("time"),
                    "longitude":("longitude"),
                    "realization":("ensemble"),
                    "reftime":("time"),
                    "source":("ensemble", "string60"),
                    "time_bnd":("time","time_bnd")}
        for f in filelist:
            fileout=(f.split("/")[-1])
            ftemp=f.split(".")[0]+"_temp.nc"

            #open input and output
            ncdfin=NetCDFFile(f, "a")
            ncdfout=NetCDFFile(ftemp, "w")

            variables=ncdfin.variables.keys()
            print variables
            allDimNames = ncdfin.dimensions.keys() 
            print allDimNames
  
            ##### time ######
          
            #read value from input file
            if "reftime" in variables:
                reftimein = ncdfin.variables['reftime']
            else:
                print "reftime dimension is not in the file " + f
                sys.exit(0)
            #create the time dimension
            ncdfout.createDimension('time', None)

            #create the reftime variable
            reftime=ncdfout.createVariable("reftime", "i", ('time',))
            reftime[:] = reftimein
            reftime.units = reftimein.units
            reftime.standard_name = "forecast_reference_time"
            reftime.longname = "forecast reference time"
            reftime.calendar=reftimein.calendar
       

            #create the leadtime variable
            leadtime=ncdfout.createVariable("leadtime", "i", ('time',))
            #to be changed?
            leadtime[:] = [20209, 20209, 20209, 20209, 20209, 20209, 20209]
            leadtime.units = "hours" 
            leadtime.standard_name = "forecast_period"
            leadtime.longname = "Time elapsed since the start of the forecast"
            leadtime.bounds = "time_bnd"

            ##### longitude - latitude ######

            #read value from input file
            if 'longitude' in variables:
                lonin = ncdfin.variables['longitude']
            else:
                print "longitude dimension is not in the file " + f
                sys.exit(0)
            if 'latitude' in variables:
                latin = ncdfin.variables['latitude']
            else:
                print "latitude dimension is not in the file " + f
                sys.exit(0)

            #create the spatial dimension
            ncdfout.createDimension('longitude', shape(lonin)[0])
            ncdfout.createDimension('latitude', shape(latin)[0])

            #create the longitude variable
            lon=ncdfout.createVariable("longitude", "f", ('longitude',))
            lat=ncdfout.createVariable("latitude", "f", ('latitude',))
            lon.data_type = "float"
            lat.data_type = "float"
            lon.units = "degrees_east"
            lat.units = "degrees_north"
            lon.axis = "X"
            lat.axis = "Y"
            lon.standard_name = "longitude"
            lat.standard_name = "latitude"
            lon.topology = "circular" 
            lon.modulo = 360
            lon.valid_min = 0.
            lat.valid_min = -89.
            lon.valid_max = 359.
            lat.valid_max = 89. 
            lon[:]=lonin
            lat[:]=latin

            ##### ensembles ######
            
            print nmember

            ncdfout.createDimension('ensemble', nmember)

            ###### experiment_id ######


            ncdfout.close()
            sys.exit(0)
            
 
    #dimname=var.dimensions
    #check if the dimension names are ok 
    #dimvalue=file.dimensions['ensemble']
    #dimvalue=file.dimensions['time']
    #print dimname
    #print dimvalue
    #print variables #file.variables["tasmax"][:]
    #data = ncfile.variables['data'][:]
    #variables=file2.variables.keys()
